const readline = require('readline');
const read = require('fs-readdir-recursive');
const fs = require('fs');

const reader = readline.createInterface({
  input: process.stdin,
  output: process.stdout,
});

console.log(
  `\nInnan vi forsätter med uppgraderingen till version 20.0.X, måste du se till
    - att du står i rotmappen av projektet du vill uppgradera.
    - att du inte har några ändringar som inte är committade i din branch.

  Obs: Detta script endast går igenom HTML-filerna i ditt projekt,
       finns det ändra typer av filer med styleguide kod som t.ex. ts, tsx, js, 
       less, scss etc. så får ni manuellt justera dessa.`
);

reader.question('\nVill du fortsätta? (y/n)', async function (svar) {
  if (svar.toLowerCase() == 'y') {
    console.log('Letar igenom alla html filer.\n');
    const htmlFiles = read(process.cwd())
      .filter(item => item.endsWith('.html'))
      .filter(item => !item.startsWith('node_modules'))
      .filter(item => !item.startsWith('bin'))
      .filter(item => !item.startsWith('dist'));

    let filesWithOldBifogaFil = [];
    let filesWithBifogaFil = [];
    let filesWithOldAccordion = [];
    let filesWithCheckboxHtmlOnly = [];
    for (const htmlFile of htmlFiles) {
      lookingLargeBtnToUpdate(htmlFile);
      lookingFlexWrapToUpdate(htmlFile);
      lookingToastrLinkToUpdate(htmlFile);
      lookingForOldBifogaFilToUpdate(htmlFile, filesWithOldBifogaFil);
      lookingForBifogaFilToUpdate(htmlFile, filesWithBifogaFil);
      lookingForOldAccordionToUpdate(htmlFile, filesWithOldAccordion);
      lookingForOldHeadersToUpdate(htmlFile);
      lookingTextbtnsToUpdate(htmlFile);
      lookingForHTMLOnlyCheckboxToUpdate(htmlFile, filesWithCheckboxHtmlOnly);
      lookingForCheckboxToUpdateNaming(htmlFile);
    }

    console.log(`
     \nManuella åtgärder:
       1. Alla knappar kommer numera inte längre ta fullbredd på skärmstorlekar mindre än 769px,
          om man vill behålla detta beteende kan du lägga in klassen "c-btn--maxwidth" som kommer
          ge knappar 100% width på små skärmar.
       2. Lägg in klassen "c-vertical-menu__list" på ul-element och "c-vertical-menu__list-item"
          på alla li-element.som ligger i <nav class="c-vertical-menu">. 
       3. Lägg in klassen "c-objectlist__list" på ul-element och "c-objectlist__list-item" på alla
          li-element.som ligger i objectlist componenten.
       4. Du behöver byta ut alla gammal komponent som använder Javascript, t.ex. "[rel="c-text-counter-js"]" 
          till webbkomponent versionen. t.ex. "<jv-bifoga-fil ... ></jv-bifoga-fil>". Gamma JS komponenter som 
          tagits bort är bifogade fil, flikar, modal, multiselect, multitext, sök, radio buttons, checkboxes, 
          checkbox-, radio button childwrapper, sidpanel och teckenräcknare.
       5. I webbkomponenten "jv-bifoga-fil" bör du använda "value" attributet för att få lista med alla bifogade filer
          Det finns även ett event av typ "valueChange" som triggas varje gång en fil läggs till eller tas bort
       6. Webbkomponenten "jv-accordion-v1" är numera borttagen och ersatt av "jv-accordion". 
          Den nya accordion har inte attributen size, heading och headingLevel utan är ersatt av en slot (referens till 
          html kod) "trigger-content-ref". 
          Du kan läsa mer på https://styleguide.jordbruksverket.se/komponenter/utfallbart-stycke.html
       7. Om du har HTML only checkbox du behöver uppdatera din HTML kod. 
          Sök med "c-checkbox__checkbox" för att hitta HTML only checkbox.
       `);

    if (filesWithOldBifogaFil.length) {
      console.log(`\nHittade manuella åtgärd nr.4: Du behöver uppdatera följande filer med gammal komponenter:`);
      for (const file of filesWithOldBifogaFil) {
        console.log('* ' + file);
      }
    }

    if (filesWithBifogaFil.length) {
      console.log(
        `\n -Hittade manuella åtgärd nr.5: Kolla om du behöver uppdatera följande filer som 
        innehåller "jv-bifoga-fil", så att du kan få lista med alla bifogade filer via "value" attributet :`
      );
      for (const file of filesWithBifogaFil) {
        console.log('* ' + file);
      }
    }

    if (filesWithOldAccordion.length) {
      console.log(
        `\nHittade manuella åtgärd nr.6: Du behöver uppdatera följande filer och ta bort depricated "jv-accordion-v1" komponenten:`
      );
      for (const file of filesWithOldAccordion) {
        console.log('* ' + file);
      }
    }

    if (filesWithCheckboxHtmlOnly.length) {
      console.log(
        `\nHittade manuella åtgärd nr.8: Du behöver uppdatera följande filer med HTML only checkbox komponenter:`
      );
      for (const file of filesWithCheckboxHtmlOnly) {
        console.log('* ' + file);
      }
    }

    await reader.close();
  } else {
    await reader.close();
  }
});

const lookingForOldAccordionToUpdate = function (htmlFile, filesWithOldAccordion) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/jv-accordion-v1/i.test(data)) {
    filesWithOldAccordion.push(htmlFile);
  }
};

const lookingForOldHeadersToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/jordbruksverket-logotype.svg\"/i.test(data)) {
    data = data.replace(/jordbruksverket-logotype.svg/g, `jordbruksverket-logotype--small.svg`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat jordbruksverket-logotype.svg och ändrat till jordbruksverket-logotype--small.svg i ' + htmlFile
    );
  }
  if (/c-header__content--title__divider\"/i.test(data)) {
    data = data.replace(/c-header__content--title__divider/g, `c-header__content__title__divider`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat c-header__content--title__divider och ändrat till c-header__content__title__divider i ' + htmlFile
    );
  }
};

const lookingForBifogaFilToUpdate = function (htmlFile, filesWithOldBifogaFil) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/jv-bifoga-fil/i.test(data)) {
    filesWithOldBifogaFil.push(htmlFile);
  }
};

const lookingTextbtnsToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');

  if (/class=".*c-btn-text--large\"/i.test(data)) {
    data = data.replace(/c-btn-text--large/g, `c-btn--medium`);
    writeToFile(htmlFile, data, '\nHittat .c-btn-text--large och ändrat till c-btn--medium i ' + htmlFile);
  }

  if (/class=".*c-btn-text--medium\"/i.test(data)) {
    data = data.replace(/c-btn-text--medium/g, `c-btn--compact`);
    writeToFile(htmlFile, data, '\nHittat .c-btn-text--medium och ändrat till c-btn--compact i ' + htmlFile);
  }

  if (/class=".*c-btn-text--small\"/i.test(data)) {
    data = data.replace(/c-btn-text--small/g, `c-btn--small`);
    writeToFile(htmlFile, data, '\nHittat .c-btn-text--small och ändrat till c-btn--small i ' + htmlFile);
  }
};

const lookingLargeBtnToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');

  if (/class=".*c-btn--large\"/i.test(data)) {
    data = data.replace(/c-btn--large/g, `c-btn--medium`);
    writeToFile(htmlFile, data, '\nHittat .c-btn--large och ändrat till c-btn--medium i ' + htmlFile);
  }

  if (/class=".*c-primary-btn--large\"/i.test(data)) {
    data = data.replace(/c-primary-btn--large/g, `c-primary-btn--medium`);
    writeToFile(htmlFile, data, '\nHittat .c-primary-btn--large och ändrat till c-primary-btn--medium i ' + htmlFile);
  }
};

const lookingForHTMLOnlyCheckboxToUpdate = function (htmlFile, filesWithHTMLOnlyCheckbox) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*?c-checkbox__checkbox.*?"/i.test(data)) {
    filesWithHTMLOnlyCheckbox.push(htmlFile);
  }
};

const lookingForCheckboxToUpdateNaming = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');

  if (/class=".*c-checkbox--small\"/i.test(data)) {
    if (!/c-checkbox\s+c-checkbox--small/i.test(data)) {
      data = data.replace(/c-checkbox--small/g, `c-checkbox c-checkbox--small`);
      writeToFile(
        htmlFile,
        data,
        '\nHittat .c-checkbox--small och ändrat till c-checkbox c-checkbox--small i ' + htmlFile
      );
    }
  }

  if (/class=".*c-checkbox--medium\"/i.test(data)) {
    data = data.replace(/c-checkbox--medium/g, `c-checkbox`);
    writeToFile(htmlFile, data, '\nHittat .c-checkbox--medium och ändrat till c-checkbox i ' + htmlFile);
  }
};

const lookingFlexWrapToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');

  if (/class=".*l-flex-wrap\"/i.test(data)) {
    data = data.replace(/l-flex-wrap/g, `l-flex--wrap`);
    writeToFile(htmlFile, data, '\nHittat .l-flex-wrap och ändrat till l-flex--wrap i ' + htmlFile);
  }

  if (/class=".*l-flex-nowrap\"/i.test(data)) {
    data = data.replace(/l-flex-nowrap/g, `l-flex--nowrap`);
    writeToFile(htmlFile, data, '\nHittat .l-flex-nowrap och ändrat till l-flex--nowrap i ' + htmlFile);
  }

  if (/class="[^"]*l-align--[^\s"]*("[^"]*|\s|$)/g.test(data)) {
    data = data.replace(/(?:\b)l-align--/g, 'l-align-');
    writeToFile(htmlFile, data, '\nHittat *l-align-- och ändrat till *l-align- i ' + htmlFile);
  }

  if (/class="[^"]*l-align-self--[^\s"]*("[^"]*|\s|$)/g.test(data)) {
    data = data.replace(/(?:\b)l-align-self--/g, 'l-align-self-');
    writeToFile(htmlFile, data, '\nHittat *l-align-self-- och ändrat till *l-align-self- i ' + htmlFile);
  }
};

const lookingForOldBifogaFilToUpdate = function (htmlFile, filesWithOldBifogaFil) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/rel="c-file-upload-js\"/i.test(data)) {
    filesWithOldBifogaFil.push(htmlFile);
  }
};

const lookingToastrLinkToUpdate = function (htmlFile) {
  let data = fs.readFileSync(htmlFile, 'utf8');
  if (/class=".*c-toaster__actionlink\"/i.test(data)) {
    data = data.replace(/c-toaster__actionlink"/g, `c-btn--medium c-btn--dark"`);
    writeToFile(
      htmlFile,
      data,
      '\nHittat .c-toaster__actionlink och ändrat till c-btn--medium c-btn--dark i ' + htmlFile
    );
  }
};

function findAndUpdate(data, nbrOfString, regexp) {
  const matchingArray = data.match(regexp);

  if (!matchingArray) {
    return { data, nbrOfString };
  }
  for (const matchingString of matchingArray) {
    if (!matchingString.includes('aria-label="')) {
      data = replaceAll(data, matchingString, matchingString.replace());
      nbrOfString = nbrOfString + 1;
    }
  }
  return { data, nbrOfString };
}

function writeToFile(htmlFile, data, message) {
  fs.writeFileSync(htmlFile, data, 'utf8');
  console.log(message);
}

reader.on('close', function () {
  console.log('\nAvslutar!');
  process.exit(0);
});

function replaceAll(string, match, replaceString) {
  return string.split(match).join(replaceString);
}
